<?php
/*
* 2007-2015 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2015 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_'))
	exit;

class CashOnDeliveryWithFee extends PaymentModule
{	
 	private	$_html = '';
	private $_postErrors = array();
	
	public function __construct()
	{
		$this->name = 'cashondeliverywithfee';
		$this->tab = 'payments_gateways';
		$this->version = '0.7.2';
		$this->author = 'PrestaShop';
		$this->need_instance = 1;
		$this->controllers = array('validation');
		$this->is_eu_compatible = 1;

		$this->currencies = true;
		$this->currencies_mode = 'checkbox';

		parent::__construct();

		$this->displayName = $this->l('Cash on delivery with fee (COD)');
		$this->description = $this->l('Accept cash on delivery payments with extra fee');

		/* For 1.4.3 and less compatibility */
		$updateConfig = array('PS_OS_CHEQUE', 'PS_OS_PAYMENT', 'PS_OS_PREPARATION', 'PS_OS_SHIPPING', 'PS_OS_CANCELED', 'PS_OS_REFUND', 'PS_OS_ERROR', 'PS_OS_OUTOFSTOCK', 'PS_OS_BANKWIRE', 'PS_OS_PAYPAL', 'PS_OS_WS_PAYMENT');
		if (!Configuration::get('PS_OS_PAYMENT'))
			foreach ($updateConfig as $u)
				if (!Configuration::get($u) && defined('_'.$u.'_'))
					Configuration::updateValue($u, constant('_'.$u.'_'));
	}

	public function install()
	{
		if (!parent::install() 
      OR !Configuration::updateValue('COD_TAX', '0') OR !Configuration::updateValue('COD_FEE', '0') OR !Configuration::updateValue('COD_FEE_TYPE', '0') OR !Configuration::updateValue('COD_FEE_MIN', '0') OR !Configuration::updateValue('COD_FEE_MAX', '0')
			OR !$this->registerHook('payment') OR ! $this->registerHook('displayPaymentEU') OR !$this->registerHook('paymentReturn'))
			return false;
		return true;
	}

	public function uninstall()
	{
		if (!Configuration::deleteByName('COD_TAX') OR !Configuration::deleteByName('COD_FEE') OR (!Configuration::deleteByName('COD_FEE_TYPE')) OR !parent::uninstall() OR (!Configuration::deleteByName('COD_FEE_MIN')) OR (!Configuration::deleteByName('COD_FEE_MAX')))
			return false;
		return true;
	}
	
	public function hasProductDownload($cart)
	{
		foreach ($cart->getProducts() AS $product)
		{
			$pd = ProductDownload::getIdFromIdProduct((int)($product['id_product']));
			if ($pd AND Validate::isUnsignedInt($pd))
				return true;
		}
		return false;
	}
	public function getContent()
	{
		$this->_html = '<h2>CASH ON DELIVERY</h2>';
		if (isset($_POST['submitCOD']))
		{
			if (empty($_POST['tax']))
				$_POST['tax'] = '0';
			if (empty($_POST['fee']))
				$_POST['fee'] = '0';
			if (empty($_POST['feetype']))
				$_POST['feetype'] = '0';
			if (empty($_POST['feemin']))
				$_POST['feemin'] = '0';
			if (empty($_POST['feemax']))
				$_POST['feemax'] = '0';
			if (!sizeof($this->_postErrors))
			{
				Configuration::updateValue('COD_TAX', floatval( $_POST['tax']) );
				Configuration::updateValue('COD_FEE', floatval( $_POST['fee']) );
				Configuration::updateValue('COD_FEE_TYPE', floatval( $_POST['feetype']) );
				Configuration::updateValue('COD_FEE_MIN', floatval( $_POST['feemin']) );
				Configuration::updateValue('COD_FEE_MAX', floatval( $_POST['feemax']) );
				$this->displayConf();
			}
			else
				$this->displayErrors();
		}

		$this->displayCOD();
		$this->displayFormSettings();
		return $this->_html;
	}
	
	public function displayCOD()
	{
		$this->_html .= '
		<img src="../modules/cashondeliverywithfee/logo.gif" style="float:left; margin-right:15px;" />
		<b>'.$this->l('This module allows you to accept payments by Cash on delivery and charge a extra fee.').'</b><br /><br /><br />';
	}
	
	
	public function displayFormSettings()
	{
		$conf = Configuration::getMultiple(array('COD_TAX','COD_FEE','COD_FEE_TYPE','COD_FEE_MIN','COD_FEE_MAX'));
		$tax = array_key_exists('tax', $_POST) ? $_POST['tax'] : (array_key_exists('COD_TAX', $conf) ? $conf['COD_TAX'] : '');
		$fee = array_key_exists('fee', $_POST) ? $_POST['fee'] : (array_key_exists('COD_FEE', $conf) ? $conf['COD_FEE'] : '');
		$feetype = array_key_exists('feetype', $_POST) ? $_POST['feetype'] : (array_key_exists('COD_FEE_TYPE', $conf) ? $conf['COD_FEE_TYPE'] : '');
		$feemin = array_key_exists('feemin', $_POST) ? $_POST['feemin'] : (array_key_exists('COD_FEE_MIN', $conf) ? $conf['COD_FEE_MIN'] : '');
		$feemax = array_key_exists('feemax', $_POST) ? $_POST['feemax'] : (array_key_exists('COD_FEE_MAX', $conf) ? $conf['COD_FEE_MAX'] : '');

		$this->_html .= '
		<form action="'.$_SERVER['REQUEST_URI'].'" method="post">
		<fieldset>
			<legend><img src="../img/admin/contact.gif" />'.$this->l('Settings').'</legend>
			<label>'.$this->l('Percentage').'</label>
			<div class="margin-form"><input type="text" size="33" name="tax" value="'.htmlentities($tax, ENT_COMPAT, 'UTF-8').'" /></div>
			<label>'.$this->l('Amount').'</label>
			<div class="margin-form"><input type="text" size="33" name="fee" value="'.htmlentities($fee, ENT_COMPAT, 'UTF-8').'" /></div>
			<br />
			<label>'.$this->l('Type').'</label>
			<div class="margin-form">
			<input type="radio" name="feetype" value="0" '.($feetype=='0' ? 'checked="checked"' : '').' /> '.$this->l('Fixed').'
			<input type="radio" name="feetype" value="1" '.($feetype=='1' ? 'checked="checked"' : '').' /> '.$this->l('Percentage').'
			<input type="radio" name="feetype" value="2" '.($feetype=='2' ? 'checked="checked"' : '').' /> '.$this->l('Fixed').'+'.$this->l('Percentage').'
			</div>
			<label>'.$this->l('Minimum Fee').'</label>
			<div class="margin-form"><input type="text" size="33" name="feemin" value="'.htmlentities($feemin, ENT_COMPAT, 'UTF-8').'" /></div>
			<label>'.$this->l('Maximum Fee').'</label>
			<div class="margin-form"><input type="text" size="33" name="feemax" value="'.htmlentities($feemax, ENT_COMPAT, 'UTF-8').'" /></div>
			<br />
			<br /><center><input type="submit" name="submitCOD" value="'.$this->l('Update settings').'" class="button" /></center>
		</fieldset>
		</form><br /><br />';
		
		$this->_html .='<fieldset class="space">
			<legend><img src="../img/admin/unknown.gif" alt="" class="middle" />Info</legend>
			 <h4>'.$this->l('User instructions').'</h4>
			 <ol>
				<li>'.$this->l('Set your choice in radio buttons (fixed tax, percentage tax or both).').'</li>
			 	<li>'.$this->l('Set the value in the amount box and/or in the percentage box, depends of your choice.').'</li>
			 	<li>'.$this->l('If you wan\'t a minimum value for percentage or fixed plus percentage fees, enter the value in minimum fee box.').'</li>
			 	<li>'.$this->l('If you wan\'t a maximum value for percentage or fixed plus percentage fees, enter the value in maximum fee box.').'</li>
			 	<li>'.$this->l('If minimum fee is higher than maximum fee maximum fee doesn\'t works.').'</li>
			 	<li>'.$this->l('To deactivate the minimum and maximum fees set it to 0 (zero) value.').'</li>
			</ol>
			<h4>'.$this->l('Credits').'</h4>
			'.$this->l('Modified fixed and/or percentaje taxes with minimal and maximal values (v0.6) module by Santos Aranda Mateos for ').'<a href="http://www.ps3pic.es" title="PS3Pic" alt="www.ps3pic.es">PS3Pic</a>.
			<br />
			'.$this->l('Original fixed or percentaje taxes with minimum value (v0.4) module by ').'<a href="http://www.prestaworks.com" title="Prestaworks" alt="www.prestaworks.com">Prestaworks</a>.
		</fieldset>';
	}

public function displayConf()
	{
		$this->_html .= '
		<div class="conf confirm">
			<img src="../img/admin/ok.gif" alt="'.$this->l('Confirmation').'" />
			'.$this->l('Settings updated').'
		</div>';
	}
	
	public function displayErrors()
	{
		$nbErrors = sizeof($this->_postErrors);
		$this->_html .= '
		<div class="alert error">
			<h3>'.($nbErrors > 1 ? $this->l('There are') : $this->l('There is')).' '.$nbErrors.' '.($nbErrors > 1 ? $this->l('errors') : $this->l('error')).'</h3>
			<ol>';
		foreach ($this->_postErrors AS $error)
			$this->_html .= '<li>'.$error.'</li>';
		$this->_html .= '
			</ol>
		</div>';
	}

	public function hookPayment($params)
	{
		if (!$this->active)
			return ;

		global $smarty;

		// Check if cart has product download
		if ($this->hasProductDownload($params['cart']))
			return false;


		$smarty->assign(array(
			'this_path' => $this->_path, //keep for retro compat
			'fee' => Tools::convertPrice(number_format($this->getCost($params), 2, '.', ''), $this->context->cart->id_currency),
			'this_path_cod' => $this->_path,
			'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/'
		));
		return $this->display(__FILE__, 'payment.tpl');
	}
	
	//Return the fee cost
	function getCost($params)
	{
	if(Configuration::get('COD_FEE_TYPE')==0)
	{
		return floatval(Configuration::get('COD_FEE'));
	}
	else if(Configuration::get('COD_FEE_TYPE')==1)
	{
		$minimalfee = floatval(Configuration::get('COD_FEE_MIN'));
		$maximalfee = floatval(Configuration::get('COD_FEE_MAX'));
		$cartvalue = floatval($params['cart']->getOrderTotal(true, 3));
		$percent = floatval(Configuration::get('COD_TAX'));
		$percent = $percent / 100;
		$fee = $cartvalue * $percent;
		
		if(($fee<$minimalfee)&($minimalfee!=0))
		{
			$fee=$minimalfee;
		}
		else if (($fee>$maximalfee)&($maximalfee!=0))
		{
			$fee=$maximalfee;
		}
		return floatval($fee);
	}
	else
	{
		$minimalfee = floatval(Configuration::get('COD_FEE_MIN'));
		$maximalfee = floatval(Configuration::get('COD_FEE_MAX'));
		$cartvalue = floatval($params['cart']->getOrderTotal(true, 3));
		$percent = floatval(Configuration::get('COD_TAX'));
		$percent = $percent / 100;
		$tax = floatval(Configuration::get('COD_FEE'));
		$fee = ($cartvalue * $percent) + $tax;
		
		if(($fee<$minimalfee)&($minimalfee!=0))
		{
			$fee=$minimalfee;
		}
		else if (($fee>$maximalfee)&($maximalfee!=0))
		{
			$fee=$maximalfee;
		}
		return floatval($fee);
	}
	}
	
	//Return the fee cost
	function getCostValidated($cart)
	{
	if(Configuration::get('COD_FEE_TYPE')==0)
	{
		return floatval(Configuration::get('COD_FEE'));
	}
	else if(Configuration::get('COD_FEE_TYPE')==1)
	{
		$minimalfee = floatval(Configuration::get('COD_FEE_MIN'));
		$maximalfee = floatval(Configuration::get('COD_FEE_MAX'));
		$cartvalue = floatval($cart->getOrderTotal(true, 3));
		$percent = floatval(Configuration::get('COD_TAX'));
		$percent = $percent / 100;
		$fee = $cartvalue * $percent;
		
		if(($fee<$minimalfee)&($minimalfee!=0))
		{
			$fee=$minimalfee;
		}
		else if (($fee>$maximalfee)&($maximalfee!=0))
		{
			$fee=$maximalfee;
		}
		return floatval($fee);
	}
	else
	{
		$minimalfee = floatval(Configuration::get('COD_FEE_MIN'));
		$maximalfee = floatval(Configuration::get('COD_FEE_MAX'));
		$cartvalue = floatval($params['cart']->getOrderTotal(true, 3));
		$percent = floatval(Configuration::get('COD_TAX'));
		$percent = $percent / 100;
		$tax = floatval(Configuration::get('COD_FEE'));
		$fee = ($cartvalue * $percent) + $tax;
		
		if(($fee<$minimalfee)&($minimalfee!=0))
		{
			$fee=$minimalfee;
		}
		else if (($fee>$maximalfee)&($maximalfee!=0))
		{
			$fee=$maximalfee;
		}
		return floatval($fee);
	}
	}

	public function hookDisplayPaymentEU($params)
	{
		if (!$this->active)
			return ;

		// Check if cart has product download
		if ($this->hasProductDownload($params['cart']))
			return false;

		return array(
			'cta_text' => $this->l('Pay with cash on delivery  with fee (COD)'),
			'logo' => Media::getMediaPath(dirname(__FILE__).'/cashondelivery.png'),
			'action' => $this->context->link->getModuleLink($this->name, 'validation', array('confirm' => true), true)
		);
	}

	public function hookPaymentReturn($params)
	{
		if (!$this->active)
			return ;

		return $this->display(__FILE__, 'confirmation.tpl');
	}
}
